import os
import subprocess
import tempfile
import time
import pathlib
from urllib.parse import quote, urlparse
from loguru import logger
import random
from typing import Optional, List


class ImageUploadError(Exception):
    """이미지 업로드 중 발생하는 예외를 처리하기 위한 커스텀 예외 클래스"""


class ImageUploader:
    """이미지 업로드를 위한 추상 기본 클래스"""

    def _upload_to_host(self, file_path: str) -> str:
        """
        호스팅 서비스에 이미지를 업로드합니다.

        Args:
            file_path (str): 업로드할 이미지 파일 경로

        Returns:
            str: 업로드된 이미지의 URL

        Raises:
            ImageUploadError: 업로드 실패시 발생
        """
        return upload_image(file_path)

    def upload(self, source: str, max_retries: int = 3) -> str:
        """
        이미지를 업로드하고 URL을 반환합니다.

        Args:
            source (str): 이미지 파일 경로 또는 URL
            max_retries (int): URL 다운로드 시 최대 재시도 횟수

        Returns:
            str: 업로드된 이미지의 URL

        Raises:
            ImageUploadError: 업로드 실패시 발생
        """
        if not source:
            logger.error("소스가 비어있습니다.")
            raise ImageUploadError("소스가 비어있습니다.")

        # URL인지 파일 경로인지 확인
        is_url = bool(urlparse(source).scheme)

        try:
            if is_url:
                logger.info(f"URL에서 이미지 업로드 시도: {source}")
                return self._upload_from_url(source, max_retries)
            else:
                logger.info(f"파일에서 이미지 업로드 시도: {source}")
                return self._upload_from_file(source)
        except Exception as e:
            logger.error(f"이미지 업로드 실패: {str(e)}")
            raise ImageUploadError(f"이미지 업로드 실패: {str(e)}")

    def _upload_from_file(self, file_path: str) -> str:
        """파일에서 직접 업로드"""
        if not os.path.exists(file_path):
            logger.error(f"파일을 찾을 수 없습니다: {file_path}")
            raise ImageUploadError(f"파일을 찾을 수 없습니다: {file_path}")

        logger.info(f"파일 업로드 중: {file_path}")
        return self._upload_to_host(file_path)

    def _upload_from_url(self, url: str, max_retries: int) -> str:
        """URL에서 이미지를 다운로드하고 업로드"""
        # URL 인코딩
        parsed_url = urlparse(url)
        encoded_path = quote(parsed_url.path)
        encoded_url = f"{parsed_url.scheme}://{parsed_url.netloc}{encoded_path}"

        # 임시 파일 생성
        file_ext = os.path.splitext(parsed_url.path)[1].lower() or ".jpg"
        temp_file = tempfile.NamedTemporaryFile(suffix=file_ext, delete=False)
        temp_path = temp_file.name
        temp_file.close()

        try:
            logger.info(f"이미지 다운로드 중: {encoded_url}")
            self._download_image(encoded_url, temp_path, max_retries)
            logger.info(f"임시 파일에서 업로드 중: {temp_path}")
            return self._upload_to_host(temp_path)
        finally:
            # 임시 파일 정리
            try:
                if os.path.exists(temp_path):
                    os.unlink(temp_path)
                    logger.info(f"임시 파일 삭제 완료: {temp_path}")
            except OSError as e:
                logger.warning(f"임시 파일 삭제 실패: {str(e)}")

    def _download_image(self, url: str, temp_path: str, max_retries: int) -> None:
        """이미지 다운로드"""
        curl_command = [
            "curl",
            "-L",
            "-f",
            "-S",
            "-s",
            "--connect-timeout",
            "30",
            "--max-time",
            "60",
            "-A",
            "Mozilla/5.0 (Windows NT 10.0; Win64; x64)",
            "-o",
            temp_path,
            url,
        ]

        for attempt in range(max_retries):
            try:
                logger.info(f"다운로드 시도 {attempt + 1}/{max_retries}: {url}")
                result = subprocess.run(curl_command, capture_output=True, text=True)
                if result.returncode == 0:
                    logger.info("다운로드 성공")
                    break

                if attempt == max_retries - 1:
                    logger.error(f"Curl 다운로드 실패: {result.stderr}")
                    raise ImageUploadError(f"Curl 다운로드 실패: {result.stderr}")

                time.sleep(2**attempt)
            except Exception as e:
                logger.error(f"이미지 다운로드 실패: {str(e)}")
                if attempt == max_retries - 1:
                    raise ImageUploadError(f"이미지 다운로드 실패: {str(e)}")
                continue

        if os.path.getsize(temp_path) < 100:
            logger.error("다운로드된 파일이 너무 작습니다")
            raise ImageUploadError("다운로드된 파일이 너무 작습니다")


def get_available_services() -> List[str]:
    """사용 가능한 이미지 업로드 서비스 목록을 반환합니다."""
    return "anhmoe beeimg fastpic imagebin pixhost sxcu".split()


def normalize_url(url: str) -> str:
    """
    URL을 정규화합니다. 중복된 'https:'를 제거하고 올바른 형식으로 변환합니다.

    Args:
        url (str): 정규화할 URL

    Returns:
        str: 정규화된 URL
    """
    # 중복된 'https:'를 처리
    if url.startswith("https:https://"):
        url = "https://" + url.split("https://")[-1]
    return url


def upload_image(file_path: str, service: Optional[str] = None) -> str:
    """
    지정된 서비스 또는 랜덤하게 선택된 서비스를 사용하여 이미지를 업로드합니다.

    Args:
        file_path (str): 업로드할 이미지 파일 경로
        service (Optional[str]): 사용할 서비스 이름. None이면 랜덤 선택

    Returns:
        str: 업로드된 이미지의 URL

    Raises:
        ValueError: 잘못된 서비스 이름이 지정된 경우
        ImageUploadError: 업로드 실패시
    """
    available_services = get_available_services()
    if service is None:
        service = random.choice(available_services)
    elif service not in available_services:
        raise ValueError(f"지원하지 않는 서비스입니다. 사용 가능한 서비스: {', '.join(available_services)}")

    logger.info(f"filepath: {file_path}, current_dir: {os.getcwd()}")
    logger.info(f"exists file: {os.path.exists(file_path)}")
    logger.info(f"Uploading image using {service} service...")

    # file_path 를 pathlib을 이용해서 존재를 확인하고, 절대경로로 변환
    if not os.path.exists(file_path):
        raise FileNotFoundError(f"파일을 찾을 수 없습니다: {file_path}")
    # 절대경로를 스트링으로 변환
    file_path = str(pathlib.Path(file_path).absolute())
    logger.info(f"absolute file_path: {file_path}")

    cmd = ["poetry", "run", "images-upload-cli", "-h", service, file_path]
    cmd_str = " ".join(cmd)
    logger.info(f"cmd: {cmd_str}")

    try:
        # shell=True로 설정하여 시스템 셸을 통해 실행
        process = subprocess.Popen(
            cmd_str,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            shell=True,
            text=True,
            universal_newlines=True,
            env=os.environ.copy(),  # 현재 환경변수 복사
        )

        # 실시간으로 출력 처리
        stdout_lines = []
        while True:
            output = process.stdout.readline()
            if output == "" and process.poll() is not None:
                break
            if output:
                line = output.strip()
                logger.info(f"Upload progress: {line}")
                stdout_lines.append(line)

        # 프로세스 완료 대기 및 에러 확인
        return_code = process.wait()
        stderr = process.stderr.read()

        if return_code != 0:
            logger.error(f"Upload failed with error: {stderr}")
            raise ImageUploadError(f"이미지 업로드 실패: {stderr}")

        # 마지막 줄이 URL일 것으로 예상
        if stdout_lines:
            url = stdout_lines[-1]
            # URL 정규화
            normalized_url = normalize_url(url)
            if normalized_url != url:
                logger.info(f"URL 정규화: {url} -> {normalized_url}")
            return normalized_url
        else:
            raise ImageUploadError("업로드는 성공했으나 URL을 받지 못했습니다.")

    except Exception as e:
        logger.error(f"Upload process failed: {str(e)}")
        raise ImageUploadError(f"이미지 업로드 프로세스 실패: {str(e)}")
