import pyimgur
import requests
from loguru import logger

from .image_uploader import ImageUploader, ImageUploadError

CLIENT_ID = "38875760a79e549"


class ImgurUploader(ImageUploader):
    """Imgur를 사용한 이미지 업로더 구현체"""

    def __init__(self, client_id: str = CLIENT_ID):
        """
        ImgurUploader 클래스의 생성자입니다.

        Args:
            client_id (str): Imgur API 클라이언트 ID. 기본값은 상수 CLIENT_ID입니다.
        """
        self.client_id = client_id
        logger.info("ImgurUploader 초기화됨: client_id 설정 완료")

    def _upload_to_host(self, file_path: str) -> str:
        """
        Imgur API를 통해 이미지를 업로드하는 메서드입니다.

        Args:
            file_path (str): 업로드할 이미지 파일의 경로.

        Returns:
            str: 업로드된 이미지의 URL.

        Raises:
            ImageUploadError: 업로드 실패 시 발생.
        """
        logger.info(f"파일 업로드 시작: {file_path}")
        try:
            # pyimgur를 사용한 업로드 시도
            im = pyimgur.Imgur(self.client_id)
            uploaded_image = im.upload_image(file_path, title="SceneStealer")
            logger.info(f"파일 업로드 성공: {uploaded_image.link}")
            return uploaded_image.link
        except Exception as e:
            logger.error(f"pyimgur 업로드 실패: {str(e)}")
            # pyimgur 실패 시 직접 API 호출
            imgur_url = "https://api.imgur.com/3/image"
            headers = {"Authorization": f"Client-ID {self.client_id}"}

            with open(file_path, "rb") as image_file:
                files = {"image": image_file}
                response = requests.post(
                    imgur_url, headers=headers, files=files, timeout=30
                )

                if response.status_code != 200:
                    logger.error(
                        f"Imgur API 오류: {response.status_code} - {response.text}"
                    )
                    raise ImageUploadError(
                        f"Imgur API 오류: {response.status_code} - {response.text}"
                    )

                result = response.json()
                if not result.get("success"):
                    logger.error(
                        f"Imgur 업로드 실패: {result.get('data', {}).get('error', '알 수 없는 오류')}"
                    )
                    raise ImageUploadError(
                        f"Imgur 업로드 실패: {result.get('data', {}).get('error', '알 수 없는 오류')}"
                    )

                logger.info(f"파일 업로드 성공: {result['data']['link']}")
                return result["data"]["link"]


# 기존 코드와의 호환성을 위한 함수들
def upload_image_link(url: str, max_retries: int = 3) -> str:
    """
    주어진 URL로부터 이미지를 업로드하는 함수입니다. 레거시 지원을 위해 제공됩니다.

    Args:
        url (str): 업로드할 이미지의 URL.
        max_retries (int): URL 다운로드 시 최대 재시도 횟수. 기본값은 3입니다.

    Returns:
        str: 업로드된 이미지의 URL.
    """
    logger.info(f"URL로부터 이미지 업로드 시도: {url}")
    uploader = FallbackImaegUploader()
    return uploader.upload(url, max_retries)


def upload_image_file(path: str) -> str:
    """
    주어진 파일 경로로부터 이미지를 업로드하는 함수입니다. 레거시 지원을 위해 제공됩니다.

    Args:
        path (str): 업로드할 이미지 파일의 경로.

    Returns:
        str: 업로드된 이미지의 URL.
    """
    logger.info(f"파일로부터 이미지 업로드 시도: {path}")
    uploader = FallbackImageUploader()
    return uploader.upload(path)
