import codecs
from loguru import logger

# 로깅 설정

from markdown2 import Markdown

from .md_converter.base import MarkdownConverter
from .imgur import upload_image_link

def convert_markdown(converter: MarkdownConverter, text: str):
    return converter.convert(text)


def convert(input_fn, output_fn, is_temp=False):
    with codecs.open(input_fn, "r", "utf_8") as fp:
        markdowner = Markdown(extras=["highlightjs-lang", "fenced-code-blocks"])
        html = markdowner.convert(fp.read())
        with codecs.open(output_fn, "w", "utf_8") as fwp:
            fwp.write(html)


def read_first_header_from_md(file_path):
    """
    마크다운 파일로부터 첫 번째 헤더를 읽어 반환하는 함수.
    :param file_path: 마크다운 파일의 경로
    :return: 첫 번째 헤더 (문자열), 헤더가 없으면 None 반환
    """
    with open(file_path, "r", encoding="utf-8") as file:
        for line in file:
            # 마크다운 헤더는 '#'으로 시작함
            if line.startswith("#"):
                return line.strip()  # 헤더 반환 전 앞뒤 공백 제거
    return None  # 파일에 헤더가 없는 경우


def upload_images_to_imgur(file_path):
    """
    마크다운 파일에 포함된 이미지들을 Imgur에 업로드하고, 마크 다운의 이미지 링크를 교체하는 함수.
    :param file_path: 마크다운 파일의 경로
    :return: 이미지 업로드 성공 여부
    """
    logger.info(f"Uploading images from markdown file: {file_path}")
    
    output = []
    with open(file_path, "r", encoding="utf-8") as file:
        lines = file.readlines()  # 모든 라인을 미리 읽어옵니다.
    
    
        for line in lines:
            if line.startswith("![") and "]" in line:
                # 이미지 링크 추출
                image_link = line.split("(")[1].split(")")[0]
                logger.info(f"Found image link: {image_link}")
                
                try:
                    # 이미지 업로드
                    uploaded_image = upload_image_link(image_link)
                    logger.info(f"Uploaded image link: {uploaded_image}")
                    
                    # 마크다운 파일에 이미지 링크 교체
                    line = line.replace(image_link, uploaded_image)
                except Exception as e:
                    logger.error(f"Error uploading image {image_link}: {e}")
                    continue  # 오류가 발생하면 다음 이미지로 넘어갑니다.

            output.append(line)

            # 교체된 라인 저장
    with open(file_path, "w", encoding="utf-8") as file:
        file.writelines(output)
            
    
    logger.info("Image upload process completed.")
    return True
